@@ -1,10 +1,11 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-__version__ = "5.0.15"
+__version__ = "5.0.22"
 import sys
 
 import gi
 import pygtkcompat
+import locale
 
 pygtkcompat.enable()
 pygtkcompat.enable_gtk(version='3.0')
@@ -198,3 +199,24 @@ if sys.platform == 'win32':
             with gtk.gdk.lock:
                 return super(Dialog, self).run()
     gtk.Dialog = Dialog
+
+if not hasattr(locale, 'localize'):
+    def localize(formatted, grouping=False, monetary=False):
+        if '.' in formatted:
+            seps = 0
+            parts = formatted.split('.')
+            if grouping:
+                parts[0], seps = locale._group(parts[0], monetary=monetary)
+            decimal_point = locale.localeconv()[
+                monetary and 'mon_decimal_point' or 'decimal_point']
+            formatted = decimal_point.join(parts)
+            if seps:
+                formatted = locale._strip_padding(formatted, seps)
+        else:
+            seps = 0
+            if grouping:
+                formatted, seps = locale._group(formatted, monetary=monetary)
+            if seps:
+                formatted = locale._strip_padding(formatted, seps)
+        return formatted
+    setattr(locale, 'localize', localize)
