@@ -140,7 +140,7 @@ def inverse_leaf(domain):
     if domain in ('AND', 'OR'):
         return domain
     elif is_leaf(domain):
-        if 'child_of' in domain[1]:
+        if 'child_of' in domain[1] and '.' not in domain[0]:
             if len(domain) == 3:
                 return domain
             else:
@@ -364,9 +364,8 @@ class Or(And):
     def inverse(self, symbol, context):
         result = []
         known_variables = set(context.keys())
-        if (symbol not in self.variables
-                and not known_variables >= self.variables):
-            # In this case we don't know anything about this OR part, we
+        if not known_variables >= (self.variables - {symbol}):
+            # In this case we don't know enough about this OR part, we
             # consider it to be True (because people will have the constraint
             # on this part later).
             return True
