@@ -10,7 +10,8 @@ import datetime
 import io
 from collections import OrderedDict
 
-from tryton.common import untimezoned_date, timezoned_date, datetime_strftime
+from tryton.common import (
+    untimezoned_date, timezoned_date, datetime_strftime, date_format)
 from tryton.common.datetime_ import date_parse
 from tryton.common.timedelta import parse as timedelta_parse
 from tryton.common.timedelta import format as timedelta_format
@@ -271,7 +272,8 @@ def convert_value(field, value, context=None):
     def convert_datetime():
         if not value:
             return
-        format_ = context.get('date_format', '%x') + ' %X'
+        format_ = (
+            date_format(context.get('date_format')) + ' ' + time_format(field))
         try:
             dt = date_parse(value, format_)
             return untimezoned_date(dt)
@@ -281,7 +283,7 @@ def convert_value(field, value, context=None):
     def convert_date():
         if not value:
             return
-        format_ = context.get('date_format', '%x')
+        format_ = date_format(context.get('date_format'))
         try:
             return date_parse(value, format_).date()
         except (ValueError, TypeError):
@@ -509,7 +511,8 @@ def format_value(field, value, target=None, context=None):
             digit = len(str(value * factor).rstrip('0').split('.')[1])
         except IndexError:
             digit = 0
-        return locale.format('%.*f', (digit, value * factor or 0), True)
+        return locale.localize(
+            '{0:.{1}f}'.format(value * factor or 0, digit), True)
 
     def format_selection():
         selections = dict(field['selection'])
@@ -524,7 +527,8 @@ def format_value(field, value, target=None, context=None):
     def format_datetime():
         if not value:
             return ''
-        format_ = context.get('date_format', '%x') + ' ' + time_format(field)
+        format_ = (
+            date_format(context.get('date_format')) + ' ' + time_format(field))
         if not isinstance(value, datetime.datetime):
             time = datetime.datetime.combine(value, datetime.time.min)
         else:
@@ -536,7 +540,7 @@ def format_value(field, value, target=None, context=None):
     def format_date():
         if not value:
             return ''
-        format_ = context.get('date_format', '%x')
+        format_ = date_format(context.get('date_format'))
         return datetime_strftime(value, format_)
 
     def format_time():
@@ -1251,7 +1255,7 @@ class DomainParser(object):
                         operator = negate_operator(default_operator(field))
                     if field['type'] in ('integer', 'float', 'numeric',
                             'datetime', 'date', 'time'):
-                        if value and '..' in value:
+                        if isinstance(value, str) and '..' in value:
                             lvalue, rvalue = value.split('..', 1)
                             lvalue = convert_value(field, lvalue, self.context)
                             rvalue = convert_value(field, rvalue, self.context)
