@@ -1,9 +1,12 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 "Form"
 import gettext
 import gtk
 import gobject
+
+from gi.repository import Gtk
+
 from tryton.gui.window.view_form.screen import Screen
 from tryton.action import Action
 from tryton.gui import Main
@@ -11,16 +14,16 @@ from tryton.gui.window import Window
 from tryton.gui.window.win_export import WinExport
 from tryton.gui.window.win_import import WinImport
 from tryton.gui.window.attachment import Attachment
+from tryton.gui.window.note import Note
 from tryton.gui.window.revision import Revision
 from tryton.signal_event import SignalEvent
-from tryton.common import message, sur, sur_3b, COLOR_SCHEMES, timezoned_date
+from tryton.common import message, sur, sur_3b, timezoned_date
 import tryton.common as common
-from tryton.translate import date_format
 from tryton.common import RPCExecute, RPCException
 from tryton.common.datetime_strftime import datetime_strftime
 from tryton import plugins
 
-from tabcontent import TabContent
+from .tabcontent import TabContent
 
 _ = gettext.gettext
 
@@ -28,118 +31,23 @@ _ = gettext.gettext
 class Form(SignalEvent, TabContent):
     "Form"
 
-    toolbar_def = [
-        ('new', 'tryton-new', _('New'), _('Create a new record'),
-            'sig_new'),
-        ('save', 'tryton-save', _('Save'), _('Save this record'),
-            'sig_save'),
-        ('switch', 'tryton-fullscreen', _('Switch'), _('Switch view'),
-            'sig_switch'),
-        ('reload', 'tryton-refresh', _('_Reload'), _('Reload'),
-            'sig_reload'),
-        (None,) * 5,
-        ('previous', 'tryton-go-previous', _('Previous'),
-            _('Previous Record'), 'sig_previous'),
-        ('next', 'tryton-go-next', _('Next'), _('Next Record'),
-            'sig_next'),
-        (None,) * 5,
-        ('attach', 'tryton-attachment', _('Attachment(0)'),
-            _('Add an attachment to the record'), 'sig_attach'),
-    ]
-    menu_def = [
-        (_('_New'), 'tryton-new', 'sig_new', '<tryton>/Form/New'),
-        (_('_Save'), 'tryton-save', 'sig_save', '<tryton>/Form/Save'),
-        (_('_Switch View'), 'tryton-fullscreen', 'sig_switch',
-            '<tryton>/Form/Switch View'),
-        (_('_Reload/Undo'), 'tryton-refresh', 'sig_reload',
-            '<tryton>/Form/Reload'),
-        (_('_Duplicate'), 'tryton-copy', 'sig_copy',
-            '<tryton>/Form/Duplicate'),
-        (_('_Delete...'), 'tryton-delete', 'sig_remove',
-            '<tryton>/Form/Delete'),
-        (None,) * 4,
-        (_('_Previous'), 'tryton-go-previous', 'sig_previous',
-            '<tryton>/Form/Previous'),
-        (_('_Next'), 'tryton-go-next', 'sig_next', '<tryton>/Form/Next'),
-        (_('_Search'), 'tryton-find', 'sig_search', '<tryton>/Form/Search'),
-        (_('View _Logs...'), None, 'sig_logs', None),
-        (_('Show revisions...'), 'tryton-clock', 'revision', None),
-        (None,) * 4,
-        (_('_Close Tab'), 'tryton-close', 'sig_win_close',
-            '<tryton>/Form/Close'),
-        (None,) * 4,
-        (_('A_ttachments...'), 'tryton-attachment', 'sig_attach',
-            '<tryton>/Form/Attachments'),
-        (_('_Actions...'), 'tryton-executable', 'sig_action',
-            '<tryton>/Form/Actions'),
-        (_('_Relate...'), 'tryton-go-jump', 'sig_relate',
-            '<tryton>/Form/Relate'),
-        (None,) * 4,
-        (_('_Report...'), 'tryton-print-open', 'sig_print_open',
-            '<tryton>/Form/Report'),
-        (_('_E-Mail...'), 'tryton-print-email', 'sig_print_email',
-            '<tryton>/Form/Email'),
-        (_('_Print...'), 'tryton-print', 'sig_print',
-            '<tryton>/Form/Print'),
-        (None,) * 4,
-        (_('_Export Data...'), 'tryton-save-as', 'sig_save_as',
-            '<tryton>/Form/Export Data'),
-        (_('_Import Data...'), None, 'sig_import',
-            '<tryton>/Form/Import Data'),
-    ]
-
-    def __init__(self, model, res_id=False, domain=None, order=None, mode=None,
-            view_ids=None, context=None, name=False, limit=None,
-            search_value=None, tab_domain=None):
-        super(Form, self).__init__()
-
-        if not mode:
-            mode = ['tree', 'form']
-        if domain is None:
-            domain = []
-        if view_ids is None:
-            view_ids = []
+    def __init__(self, model, res_id=None, name='', **attributes):
+        super(Form, self).__init__(**attributes)
 
         self.model = model
         self.res_id = res_id
-        self.domain = domain
-        self.mode = mode
-        self.context = context
-        self.view_ids = view_ids
+        self.mode = attributes.get('mode')
+        self.view_ids = attributes.get('view_ids')
         self.dialogs = []
 
-        self.screen = Screen(self.model, mode=mode, context=context,
-            view_ids=view_ids, domain=domain, limit=limit, order=order,
-            search_value=search_value, tab_domain=tab_domain)
+        self.screen = Screen(self.model, **attributes)
         self.screen.widget.show()
 
-        if not name:
-            self.name = self.screen.current_view.title
-        else:
-            self.name = name
-
-        if self.model not in common.MODELHISTORY:
-            self.menu_def = self.menu_def[:]
-            # Remove callback to revision
-            self.menu_def[11] = (self.menu_def[11][:2] + (None,)
-                + self.menu_def[11][3:])
+        self.name = name
 
         self.create_tabcontent()
 
-        self.url_entry = url_entry = gtk.Entry()
-        url_entry.show()
-        url_entry.set_editable(False)
-        style = url_entry.get_style()
-        url_entry.modify_bg(gtk.STATE_ACTIVE,
-            style.bg[gtk.STATE_INSENSITIVE])
-        self.widget.pack_start(url_entry, False, False)
-
-        access = common.MODELACCESS[self.model]
-        for button, access_type in (
-                ('new', 'create'),
-                ('save', 'write'),
-                ):
-            self.buttons[button].props.sensitive = access[access_type]
+        self.set_buttons_sensitive()
 
         self.screen.signal_connect(self, 'record-message',
             self._record_message)
@@ -149,9 +57,10 @@ class Form(SignalEvent, TabContent):
         self.screen.signal_connect(self, 'record-saved', self._record_saved)
         self.screen.signal_connect(self, 'attachment-count',
                 self._attachment_count)
+        self.screen.signal_connect(self, 'unread-note', self._unread_note)
 
         if res_id not in (None, False):
-            if isinstance(res_id, (int, long)):
+            if isinstance(res_id, int):
                 res_id = [res_id]
             self.screen.load(res_id)
         else:
@@ -173,36 +82,67 @@ class Form(SignalEvent, TabContent):
     def widget_get(self):
         return self.screen.widget
 
-    def __eq__(self, value):
-        if not value:
+    def compare(self, model, attributes):
+        if not attributes:
             return False
-        if not isinstance(value, Form):
-            return False
-        return (self.model == value.model
-            and self.res_id == value.res_id
-            and self.domain == value.domain
-            and self.mode == value.mode
-            and self.view_ids == value.view_ids
-            and self.screen.context == value.screen.context
-            and self.name == value.name
-            and self.screen.limit == value.screen.limit
-            and self.screen.search_value == value.screen.search_value)
+        return (self.model == model
+            and self.res_id == attributes.get('res_id')
+            and self.attributes.get('domain') == attributes.get('domain')
+            and self.attributes.get('view_ids') == attributes.get('view_ids')
+            and (attributes.get('view_ids')
+                or (self.attributes.get('mode') or ['tree', 'form']) == (
+                    attributes.get('mode') or ['tree', 'form']))
+            and self.screen.local_context == attributes.get('context')
+            and self.attributes.get('search_value') == (
+                attributes.get('search_value')))
+
+    def __hash__(self):
+        return id(self)
 
     def destroy(self):
+        super(Form, self).destroy()
+        self.screen.signal_unconnect(self)
         self.screen.destroy()
 
-    def ids_get(self):
-        return self.screen.ids_get()
-
-    def id_get(self):
-        return self.screen.id_get()
-
     def sig_attach(self, widget=None):
-        record = self.screen.current_record
-        if not record or record.id < 0:
+        def window(widget):
+            return Attachment(
+                record, lambda: self.update_attachment_count(reload=True))
+
+        def add_file(widget):
+            filenames = common.file_selection(_("Select"), multi=True)
+            if filenames:
+                attachment = window(widget)
+                for filename in filenames:
+                    attachment.add_file(filename)
+
+        def activate(widget, callback):
+            callback()
+
+        button = self.buttons['attach']
+        if widget != button:
+            if button.props.sensitive:
+                button.props.active = True
             return
-        Attachment(record,
-            lambda: self.update_attachment_count(reload=True))
+        record = self.screen.current_record
+        menu = button._menu = Gtk.Menu()
+        for name, callback in Attachment.get_attachments(record):
+            item = Gtk.MenuItem()
+            item.set_label(name)
+            item.connect('activate', activate, callback)
+            menu.add(item)
+        menu.add(Gtk.SeparatorMenuItem())
+        add_item = Gtk.MenuItem()
+        add_item.set_label(_("Add..."))
+        add_item.connect('activate', add_file)
+        menu.add(add_item)
+        manage_item = Gtk.MenuItem()
+        manage_item.set_label(_("Manage..."))
+        manage_item.connect('activate', window)
+        menu.add(manage_item)
+        menu.show_all()
+        menu.connect('deactivate', self._popup_menu_hide, button)
+        self.action_popup(button)
 
     def update_attachment_count(self, reload=False):
         record = self.screen.current_record
@@ -216,12 +156,45 @@ class Form(SignalEvent, TabContent):
         label = _('Attachment(%d)') % signal_data
         self.buttons['attach'].set_label(label)
         if signal_data:
-            self.buttons['attach'].set_stock_id('tryton-attachment-hi')
+            # FIXME
+            icon = 'tryton-attach'
         else:
-            self.buttons['attach'].set_stock_id('tryton-attachment')
-        record_id = self.id_get()
+            icon = 'tryton-attach'
+        image = common.IconFactory.get_image(icon, gtk.ICON_SIZE_LARGE_TOOLBAR)
+        image.show()
+        self.buttons['attach'].set_icon_widget(image)
+        record = self.screen.current_record
         self.buttons['attach'].props.sensitive = bool(
-            record_id >= 0 and record_id is not False)
+            record.id >= 0 if record else False)
+
+    def sig_note(self, widget=None):
+        record = self.screen.current_record
+        if not record or record.id < 0:
+            return
+        Note(record,
+            lambda: self.update_unread_note(reload=True))
+
+    def update_unread_note(self, reload=False):
+        record = self.screen.current_record
+        if record:
+            unread = record.get_unread_note(reload=reload)
+        else:
+            unread = 0
+        self._unread_note(None, unread)
+
+    def _unread_note(self, widget, signal_data):
+        label = _('Note(%d)') % signal_data
+        self.buttons['note'].set_label(label)
+        if signal_data:
+            self.buttons['note'].set_stock_id('tryton-note-hi')
+        else:
+            self.buttons['note'].set_stock_id('tryton-note')
+        record = self.screen.current_record
+        if not record or record.id < 0:
+            sensitive = False
+        else:
+            sensitive = True
+        self.buttons['note'].props.sensitive = sensitive
 
     def sig_switch(self, widget=None):
         if not self.modified_save():
@@ -229,9 +202,10 @@ class Form(SignalEvent, TabContent):
         self.screen.switch_view()
 
     def sig_logs(self, widget=None):
-        obj_id = self.id_get()
-        if obj_id < 0 or obj_id is False:
-            self.message_info(_('You have to select one record!'))
+        current_record = self.screen.current_record
+        if not current_record or current_record.id < 0:
+            self.message_info(
+                _('You have to select one record.'), gtk.MESSAGE_INFO)
             return False
 
         fields = [
@@ -243,25 +217,26 @@ class Form(SignalEvent, TabContent):
         ]
 
         try:
-            res = RPCExecute('model', self.model, 'read', [obj_id],
+            res = RPCExecute('model', self.model, 'read', [current_record.id],
                 [x[0] for x in fields], context=self.screen.context)
         except RPCException:
             return
+        date_format = self.screen.context.get('date_format', '%x')
+        datetime_format = date_format + ' %H:%M:%S.%f'
         message_str = ''
         for line in res:
             for (key, val) in fields:
                 value = str(line.get(key, False) or '/')
                 if line.get(key, False) \
                         and key in ('create_date', 'write_date'):
-                    display_format = date_format() + ' %H:%M:%S'
                     date = timezoned_date(line[key])
-                    value = common.datetime_strftime(date, display_format)
+                    value = common.datetime_strftime(date, datetime_format)
                 message_str += val + ' ' + value + '\n'
         message_str += _('Model:') + ' ' + self.model
         message(message_str)
         return True
 
-    def revision(self, widget=None):
+    def sig_revision(self, widget=None):
         if not self.modified_save():
             return
         current_id = (self.screen.current_record.id
@@ -272,7 +247,9 @@ class Form(SignalEvent, TabContent):
         except RPCException:
             return
         revision = self.screen.context.get('_datetime')
-        revision = Revision(revisions, revision).run()
+        format_ = self.screen.context.get('date_format', '%x')
+        format_ += ' %H:%M:%S.%f'
+        revision = Revision(revisions, revision, format_).run()
         # Prevent too old revision in form view
         if (self.screen.current_view.view_type == 'form'
                 and revision
@@ -292,15 +269,42 @@ class Form(SignalEvent, TabContent):
             self.update_revision()
 
     def update_revision(self):
+        tooltips = common.Tooltips()
         revision = self.screen.context.get('_datetime')
         if revision:
-            format_ = date_format() + ' %H:%M:%S.%f'
-            revision = datetime_strftime(revision, format_)
-            self.title.set_label('%s @ %s' % (self.name, revision))
+            format_ = self.screen.context.get('date_format', '%x')
+            format_ += ' %H:%M:%S.%f'
+            revision_label = ' @ %s' % datetime_strftime(revision, format_)
+            label = common.ellipsize(
+                self.name, 80 - len(revision_label)) + revision_label
+            tooltip = self.name + revision_label
         else:
-            self.title.set_label(self.name)
-        for button in ('new', 'save'):
-            self.buttons[button].props.sensitive = not revision
+            label = common.ellipsize(self.name, 80)
+            tooltip = self.name
+        self.title.set_label(label)
+        tooltips.set_tip(self.title, tooltip)
+        self.set_buttons_sensitive(revision)
+
+    def set_buttons_sensitive(self, revision=None):
+        if not revision:
+            access = common.MODELACCESS[self.model]
+            for name, sensitive in [
+                    ('new', access['create']),
+                    ('save', access['create'] or access['write']),
+                    ('remove', access['delete']),
+                    ('copy', access['create']),
+                    ('import', access['create']),
+                    ]:
+                if name in self.buttons:
+                    self.buttons[name].props.sensitive = sensitive
+                if name in self.menu_buttons:
+                    self.menu_buttons[name].props.sensitive = sensitive
+        else:
+            for name in ['new', 'save', 'remove', 'copy', 'import']:
+                if name in self.buttons:
+                    self.buttons[name].props.sensitive = False
+                if name in self.menu_buttons:
+                    self.menu_buttons[name].props.sensitive = False
 
     def sig_remove(self, widget=None):
         if not common.MODELACCESS[self.model]['delete']:
@@ -311,25 +315,28 @@ class Form(SignalEvent, TabContent):
             msg = _('Are you sure to remove those records?')
         if sur(msg):
             if not self.screen.remove(delete=True, force_remove=True):
-                self.message_info(_('Records not removed!'))
+                self.message_info(_('Records not removed.'), gtk.MESSAGE_ERROR)
             else:
-                self.message_info(_('Records removed!'), 'green')
+                self.message_info(_('Records removed.'), gtk.MESSAGE_INFO)
+                self.screen.count_tab_domain()
 
     def sig_import(self, widget=None):
-        while(self.screen.view_to_load):
-            self.screen.load_view_to_load()
-        fields = {}
-        for name, field in self.screen.group.fields.iteritems():
-            fields[name] = field.attrs
-        WinImport(self.model, self.screen.context)
+        WinImport(self.title.get_text(), self.model, self.screen.context)
 
-    def sig_save_as(self, widget=None):
-        while self.screen.view_to_load:
-            self.screen.load_view_to_load()
-        fields = {}
-        for name, field in self.screen.group.fields.iteritems():
-            fields[name] = field.attrs
-        WinExport(self.model, self.ids_get(), context=self.screen.context)
+    def sig_export(self, widget=None):
+        export = WinExport(
+            self.title.get_text(), self.model,
+            [r.id for r in self.screen.selected_records],
+            context=self.screen.context)
+        for name in self.screen.current_view.get_fields():
+            type = self.screen.group.fields[name].attrs['type']
+            if type == 'selection':
+                export.sel_field(name + '.translated')
+            elif type == 'reference':
+                export.sel_field(name + '.translated')
+                export.sel_field(name + '/rec_name')
+            else:
+                export.sel_field(name)
 
     def sig_new(self, widget=None, autosave=True):
         if not common.MODELACCESS[self.model]['create']:
@@ -338,7 +345,7 @@ class Form(SignalEvent, TabContent):
             if not self.modified_save():
                 return
         self.screen.new()
-        self.message_info('')
+        self.message_info()
         self.activate_save()
 
     def sig_copy(self, widget=None):
@@ -346,41 +353,48 @@ class Form(SignalEvent, TabContent):
             return
         if not self.modified_save():
             return
-        self.screen.copy()
-        self.message_info(_('Working now on the duplicated record(s)!'),
-            'green')
+        if self.screen.copy():
+            self.message_info(_('Working now on the duplicated record(s).'),
+                gtk.MESSAGE_INFO)
+            self.screen.count_tab_domain()
 
     def sig_save(self, widget=None):
+        if widget:
+            # Called from button so we must save the tree state
+            self.screen.save_tree_state()
         if not (common.MODELACCESS[self.model]['write']
                 or common.MODELACCESS[self.model]['create']):
             return
         if self.screen.save_current():
-            self.message_info(_('Record saved!'), 'green')
+            self.message_info(_('Record saved.'), gtk.MESSAGE_INFO)
+            self.screen.count_tab_domain()
             return True
         else:
-            self.message_info(_('Invalid form!'))
+            self.message_info(self.screen.invalid_message(), gtk.MESSAGE_ERROR)
             return False
 
     def sig_previous(self, widget=None):
         if not self.modified_save():
             return
         self.screen.display_prev()
-        self.message_info('')
+        self.message_info()
         self.activate_save()
 
     def sig_next(self, widget=None):
         if not self.modified_save():
             return
         self.screen.display_next()
-        self.message_info('')
+        self.message_info()
         self.activate_save()
 
     def sig_reload(self, test_modified=True):
-        if test_modified and not self.modified_save():
+        if test_modified:
+            if not self.modified_save():
                 return False
+        else:
+            self.screen.save_tree_state(store=False)
         self.screen.cancel_current()
         set_cursor = False
-        self.screen.save_tree_state(store=False)
         record_id = (self.screen.current_record.id
             if self.screen.current_record else None)
         if self.screen.current_view.view_type != 'form':
@@ -391,8 +405,9 @@ class Form(SignalEvent, TabContent):
                     set_cursor = True
                     break
         self.screen.display(set_cursor=set_cursor)
-        self.message_info('')
+        self.message_info()
         self.activate_save()
+        self.screen.count_tab_domain()
         return True
 
     def sig_action(self, widget):
@@ -415,6 +430,10 @@ class Form(SignalEvent, TabContent):
         if self.buttons['relate'].props.sensitive:
             self.buttons['relate'].props.active = True
 
+    def sig_copy_url(self, widget):
+        if self.buttons['copy_url'].props.sensitive:
+            self.buttons['copy_url'].props.active = True
+
     def sig_search(self, widget):
         search_container = self.screen.screen_container
         if hasattr(search_container, 'search_entry'):
@@ -428,27 +447,19 @@ class Form(SignalEvent, TabContent):
             menu.popdown()
             return
 
-        def menu_position(menu):
-            parent = widget.get_toplevel()
-            parent_x, parent_y = parent.window.get_origin()
+        def menu_position(menu, data):
             widget_allocation = widget.get_allocation()
-            return (
-                widget_allocation.x + parent_x,
-                widget_allocation.y + widget_allocation.height + parent_y,
-                False
-            )
+            if hasattr(widget.window, 'get_root_coords'):
+                x, y = widget.window.get_root_coords(
+                    widget_allocation.x, widget_allocation.y)
+            else:
+                x, y = widget.window.get_origin()
+                x += widget_allocation.x
+                y += widget_allocation.y
+            return (x, y + widget_allocation.height, False)
         menu.show_all()
-        menu.popup(None, None, menu_position, 0, 0)
-
-    def message_info(self, message, color='red'):
-        if message:
-            self.info_label.set_label(message)
-            self.eb_info.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(
-                COLOR_SCHEMES.get(color, 'white')))
-            self.eb_info.show_all()
-        else:
-            self.info_label.set_label('')
-            self.eb_info.hide()
+        menu.popup(None, None, menu_position, 0, gtk.get_current_event_time(),
+            None)
 
     def _record_message(self, screen, signal_data):
         name = '_'
@@ -458,6 +469,13 @@ class Form(SignalEvent, TabContent):
                 'attach'):
             button = self.buttons[button_id]
             can_be_sensitive = getattr(button, '_can_be_sensitive', True)
+            if button_id in {'print', 'relate', 'email', 'open'}:
+                action_type = button_id
+                if button_id in {'email', 'open'}:
+                    action_type = 'print'
+                can_be_sensitive |= any(
+                    b.attrs.get('keyword', 'action') == action_type
+                    for b in screen.get_buttons())
             button.props.sensitive = (bool(signal_data[0])
                 and can_be_sensitive)
         button_switch = self.buttons['switch']
@@ -467,9 +485,8 @@ class Form(SignalEvent, TabContent):
         if signal_data[1] < signal_data[2]:
             msg += _(' of ') + str(signal_data[2])
         self.status_label.set_text(msg)
-        self.message_info('')
+        self.message_info()
         self.activate_save()
-        self.url_entry.set_text(self.screen.get_url())
 
     def _record_modified(self, screen, signal_data):
         # As it is called via idle_add, the form could have been destroyed in
@@ -487,7 +504,7 @@ class Form(SignalEvent, TabContent):
         if self.screen.modified():
             value = sur_3b(
                 _('This record has been modified\n'
-                    'do you want to save it ?'))
+                    'do you want to save it?'))
             if value == 'ok':
                 return self.sig_save(None)
             if value == 'ko':
@@ -496,13 +513,13 @@ class Form(SignalEvent, TabContent):
         return True
 
     def sig_close(self, widget=None):
-        for dialog in self.dialogs[:]:
+        for dialog in reversed(self.dialogs[:]):
             dialog.destroy()
         return self.modified_save()
 
     def _action(self, action, atype):
         action = action.copy()
-        if not self.screen.save_current():
+        if self.screen.modified() and not self.sig_save():
             return
         record_id = (self.screen.current_record.id
             if self.screen.current_record else None)
@@ -513,30 +530,31 @@ class Form(SignalEvent, TabContent):
             'id': record_id,
             'ids': record_ids,
         }
-        Action._exec_action(action, data, self.screen.context)
+        Action._exec_action(action, data, self.screen.local_context)
 
     def activate_save(self):
         self.buttons['save'].props.sensitive = self.screen.modified()
 
     def sig_win_close(self, widget):
-        Main.get_main().sig_win_close(widget)
+        Main().sig_win_close(widget)
 
     def create_toolbar(self, toolbars):
         gtktoolbar = super(Form, self).create_toolbar(toolbars)
 
         attach_btn = self.buttons['attach']
+        target_entry = gtk.TargetEntry.new('text/uri-list', 0, 0)
         attach_btn.drag_dest_set(gtk.DEST_DEFAULT_ALL, [
-                ('text/uri-list', 0, 0),
-                ], gtk.gdk.ACTION_MOVE)
+                target_entry,
+                ], gtk.gdk.ACTION_MOVE | gtk.gdk.ACTION_COPY)
         attach_btn.connect('drag_data_received',
             self.attach_drag_data_received)
 
         iconstock = {
             'print': 'tryton-print',
-            'action': 'tryton-executable',
-            'relate': 'tryton-go-jump',
-            'email': 'tryton-print-email',
-            'open': 'tryton-print-open',
+            'action': 'tryton-launch',
+            'relate': 'tryton-link',
+            'email': 'tryton-email',
+            'open': 'tryton-open',
         }
         for action_type, special_action, action_name, tooltip in (
                 ('action', 'action', _('Action'), _('Launch action')),
@@ -547,7 +565,10 @@ class Form(SignalEvent, TabContent):
                 ('print', 'print', _('Print'), _('Print report')),
         ):
             if action_type is not None:
-                tbutton = gtk.ToggleToolButton(iconstock.get(special_action))
+                tbutton = gtk.ToggleToolButton()
+                tbutton.set_icon_widget(common.IconFactory.get_image(
+                        iconstock.get(special_action),
+                        gtk.ICON_SIZE_LARGE_TOOLBAR))
                 tbutton.set_label(action_name)
                 tbutton._menu = self._create_popup_menu(tbutton,
                     action_type, toolbars[action_type], special_action)
@@ -561,14 +582,32 @@ class Form(SignalEvent, TabContent):
                 tbutton = gtk.SeparatorToolItem()
             gtktoolbar.insert(tbutton, -1)
 
+        gtktoolbar.insert(gtk.SeparatorToolItem(), -1)
+
+        url_button = gtk.ToggleToolButton()
+        url_button.set_icon_widget(
+            common.IconFactory.get_image(
+                'tryton-public', gtk.ICON_SIZE_LARGE_TOOLBAR))
+        url_button.set_label(_('_Copy URL'))
+        url_button.set_use_underline(True)
+        self.tooltips.set_tip(
+            url_button, _('Copy URL into clipboard'))
+        url_button._menu = url_menu = gtk.Menu()
+        url_menuitem = gtk.MenuItem()
+        url_menuitem.connect('activate', self.url_copy)
+        url_menu.add(url_menuitem)
+        url_menu.show_all()
+        url_menu.connect('deactivate', self._popup_menu_hide, url_button)
+        url_button.connect('toggled', self.url_set, url_menuitem)
+        url_button.connect('toggled', self.action_popup)
+        self.buttons['copy_url'] = url_button
+        gtktoolbar.insert(url_button, -1)
         return gtktoolbar
 
     def _create_popup_menu(self, widget, keyword, actions, special_action):
         menu = gtk.Menu()
         menu.connect('deactivate', self._popup_menu_hide, widget)
-
-        if keyword == 'action':
-            widget.connect('toggled', self._update_action_popup, menu)
+        widget.connect('toggled', self._update_popup_menu, menu, keyword)
 
         for action in actions:
             new_action = action.copy()
@@ -599,37 +638,67 @@ class Form(SignalEvent, TabContent):
     def _popup_menu_hide(self, menuitem, togglebutton):
         togglebutton.props.active = False
 
-    def _update_action_popup(self, tbutton, menu):
+    def _update_popup_menu(self, tbutton, menu, keyword):
         for item in menu.get_children():
             if (getattr(item, '_update_action', False)
                     or isinstance(item, gtk.SeparatorMenuItem)):
                 menu.remove(item)
 
-        buttons = self.screen.get_buttons()
-        if buttons:
+        buttons = [b for b in self.screen.get_buttons()
+            if keyword == b.attrs.get('keyword', 'action')]
+        if buttons and menu.get_children():
             menu.add(gtk.SeparatorMenuItem())
         for button in buttons:
-            menuitem = gtk.ImageMenuItem(button.attrs.get('icon'))
+            menuitem = gtk.ImageMenuItem()
             menuitem.set_label('_' + button.attrs.get('string', _('Unknown')))
             menuitem.set_use_underline(True)
+            if button.attrs.get('icon'):
+                menuitem.set_image(common.IconFactory.get_image(
+                        button.attrs['icon'], gtk.ICON_SIZE_MENU))
             menuitem.connect('activate',
                 lambda m, attrs: self.screen.button(attrs), button.attrs)
             menuitem._update_action = True
             menu.add(menuitem)
 
-        menu.add(gtk.SeparatorMenuItem())
+        kw_plugins = []
         for plugin in plugins.MODULES:
-            for name, func in plugin.get_plugins(self.model):
-                menuitem = gtk.MenuItem('_' + name)
-                menuitem.set_use_underline(True)
-                menuitem.connect('activate', lambda m, func: func({
-                            'model': self.model,
-                            'ids': [r.id
-                                for r in self.screen.selected_records],
-                            'id': self.id_get(),
-                            }), func)
-                menuitem._update_action = True
-                menu.add(menuitem)
+            for plugin_spec in plugin.get_plugins(self.model):
+                name, func = plugin_spec[:2]
+                try:
+                    plugin_keyword = plugin_spec[2]
+                except IndexError:
+                    plugin_keyword = 'action'
+                if keyword != plugin_keyword:
+                    continue
+                kw_plugins.append((name, func))
+
+        if kw_plugins:
+            menu.add(gtk.SeparatorMenuItem())
+        for name, func in kw_plugins:
+            menuitem = gtk.MenuItem('_' + name)
+            menuitem.set_use_underline(True)
+            menuitem.connect('activate', lambda m, func: func({
+                        'model': self.model,
+                        'ids': [r.id
+                            for r in self.screen.selected_records],
+                        'id': (self.screen.current_record.id
+                            if self.screen.current_record else None),
+                        }), func)
+            menuitem._update_action = True
+            menu.add(menuitem)
+
+    def url_copy(self, menuitem):
+        url = self.screen.get_url(self.name)
+        for selection in [gtk.CLIPBOARD_PRIMARY, gtk.CLIPBOARD_CLIPBOARD]:
+            clipboard = gtk.clipboard_get(selection)
+            clipboard.set_text(url, -1)
+
+    def url_set(self, button, menuitem):
+        url = self.screen.get_url(self.name)
+        size = 80
+        if len(url) > size:
+            url = url[:size // 2] + '...' + url[-size // 2:]
+        menuitem.set_label(url)
 
     def set_cursor(self):
         if self.screen:
@@ -643,7 +712,7 @@ class Form(SignalEvent, TabContent):
         win_attach = Attachment(record,
             lambda: self.update_attachment_count(reload=True))
         if info == 0:
-            for uri in selection.data.splitlines():
+            for uri in selection.get_uris():
                 # Win32 cut&paste terminates the list with a NULL character
                 if not uri or uri == '\0':
                     continue
