@@ -275,13 +275,24 @@ class Group(SignalEvent, list):
 
     @property
     def context(self):
-        ctx = rpc.CONTEXT.copy()
+        return self._get_context(local=False)
+
+    @property
+    def local_context(self):
+        return self._get_context(local=True)
+
+    def _get_context(self, local=False):
+        if not local:
+            ctx = rpc.CONTEXT.copy()
+        else:
+            ctx = {}
         if self.parent:
-            parent_context = self.parent.get_context()
+            parent_context = self.parent.get_context(local=local)
             ctx.update(parent_context)
             if self.child_name in self.parent.group.fields:
                 field = self.parent.group.fields[self.child_name]
-                ctx.update(field.get_context(self.parent, parent_context))
+                ctx.update(field.get_context(
+                        self.parent, parent_context, local=local))
         ctx.update(self._context)
         if self.parent_datetime_field:
             ctx['_datetime'] = self.parent.get_eval(
